@@ -24,12 +24,14 @@ import groovy.util.ObservableList;
 import org.codehaus.groovy.runtime.InvokerInvocationException;
 import org.gradle.api.Action;
 import org.gradle.api.AntBuilder;
+import org.gradle.api.Describable;
 import org.gradle.api.Incubating;
 import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.Project;
 import org.gradle.api.Task;
-import org.gradle.api.file.DirectoryVar;
-import org.gradle.api.file.RegularFileVar;
+import org.gradle.api.file.DirectoryProperty;
+import org.gradle.api.file.RegularFileProperty;
+import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.file.TaskFileVarFactory;
 import org.gradle.api.internal.file.TemporaryFileProvider;
 import org.gradle.api.internal.project.ProjectInternal;
@@ -38,11 +40,13 @@ import org.gradle.api.internal.tasks.ContextAwareTaskAction;
 import org.gradle.api.internal.tasks.DefaultTaskDependency;
 import org.gradle.api.internal.tasks.DefaultTaskDestroyables;
 import org.gradle.api.internal.tasks.DefaultTaskInputs;
+import org.gradle.api.internal.tasks.DefaultTaskLocalState;
 import org.gradle.api.internal.tasks.DefaultTaskOutputs;
 import org.gradle.api.internal.tasks.TaskContainerInternal;
 import org.gradle.api.internal.tasks.TaskDependencyInternal;
 import org.gradle.api.internal.tasks.TaskExecuter;
 import org.gradle.api.internal.tasks.TaskExecutionContext;
+import org.gradle.api.internal.tasks.TaskLocalStateInternal;
 import org.gradle.api.internal.tasks.TaskMutator;
 import org.gradle.api.internal.tasks.TaskStateInternal;
 import org.gradle.api.internal.tasks.execution.DefaultTaskExecutionContext;
@@ -57,6 +61,7 @@ import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.TaskDependency;
 import org.gradle.api.tasks.TaskDestroyables;
 import org.gradle.api.tasks.TaskInstantiationException;
+import org.gradle.api.tasks.TaskLocalState;
 import org.gradle.internal.Factory;
 import org.gradle.internal.logging.compatbridge.LoggingManagerInternalCompatibilityBridge;
 import org.gradle.internal.metaobject.DynamicObject;
@@ -120,8 +125,6 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     private final TaskStateInternal state;
 
-    private List<TaskValidator> validators;
-
     private final TaskMutator taskMutator;
     private ObservableList observableActionList;
     private boolean impliesSubProjects;
@@ -130,6 +133,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
     private final TaskInputsInternal taskInputs;
     private final TaskOutputsInternal taskOutputs;
     private final TaskDestroyables taskDestroyables;
+    private final TaskLocalStateInternal taskLocalState;
     private final Class<? extends Task> publicType;
     private LoggingManagerInternal loggingManager;
 
@@ -160,10 +164,13 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         finalizedBy = new DefaultTaskDependency(tasks);
         shouldRunAfter = new DefaultTaskDependency(tasks);
         services = project.getServices();
+
+        FileResolver fileResolver = project.getFileResolver();
         taskMutator = new TaskMutator(this);
-        taskInputs = new DefaultTaskInputs(project.getFileResolver(), this, taskMutator);
-        taskOutputs = new DefaultTaskOutputs(project.getFileResolver(), this, taskMutator);
-        taskDestroyables = new DefaultTaskDestroyables(project.getFileResolver(), this, taskMutator);
+        taskInputs = new DefaultTaskInputs(fileResolver, this, taskMutator);
+        taskOutputs = new DefaultTaskOutputs(fileResolver, this, taskMutator);
+        taskDestroyables = new DefaultTaskDestroyables(fileResolver, this, taskMutator);
+        taskLocalState = new DefaultTaskLocalState(fileResolver, this, taskMutator);
     }
 
     private void assertDynamicObject() {
@@ -371,12 +378,20 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     @Override
     public final void execute() {
-        getExecuter().execute(this, state, new DefaultTaskExecutionContext());
+        DeprecationLogger.nagUserOfDiscontinuedMethod("TaskInternal.execute()", getReuseTaskLogicAdvice());
+        TaskExecuter executer = DeprecationLogger.whileDisabled(new Factory<TaskExecuter>() {
+            @Override
+            public TaskExecuter create() {
+                return getExecuter();
+            }
+        });
+        executer.execute(this, state, new DefaultTaskExecutionContext());
         state.rethrowFailure();
     }
 
     @Override
     public TaskExecuter getExecuter() {
+        DeprecationLogger.nagUserOfDiscontinuedProperty("TaskInternal.executer", getReuseTaskLogicAdvice());
         if (executer == null) {
             executer = services.get(TaskExecuter.class);
         }
@@ -385,9 +400,15 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     @Override
     public void setExecuter(TaskExecuter executer) {
+        DeprecationLogger.nagUserOfDiscontinuedProperty("TaskInternal.executer", getReuseTaskLogicAdvice());
         this.executer = executer;
     }
 
+    private String getReuseTaskLogicAdvice() {
+        String reuseTaskLogicUrl = services.get(DocumentationRegistry.class).getDocumentationFor("custom_tasks", "sec:reusing_task_logic");
+        return "There are better ways to re-use task logic, see " + reuseTaskLogicUrl + ".";
+    }
+
     @Override
     public Task dependsOn(final Object... paths) {
         taskMutator.mutate("Task.dependsOn(Object...)", new Runnable() {
@@ -400,13 +421,18 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     @Override
     public Task doFirst(final Action<? super Task> action) {
+        return doFirst("doFirst {} action", action);
+    }
+
+    @Override
+    public Task doFirst(final String actionName, final Action<? super Task> action) {
         hasCustomActions = true;
         if (action == null) {
             throw new InvalidUserDataException("Action must not be null!");
         }
         taskMutator.mutate("Task.doFirst(Action)", new Runnable() {
             public void run() {
-                getTaskActions().add(0, wrap(action));
+                getTaskActions().add(0, wrap(action, actionName));
             }
         });
         return this;
@@ -414,13 +440,18 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     @Override
     public Task doLast(final Action<? super Task> action) {
+        return doLast("doLast {} action", action);
+    }
+
+    @Override
+    public Task doLast(final String actionName, final Action<? super Task> action) {
         hasCustomActions = true;
         if (action == null) {
             throw new InvalidUserDataException("Action must not be null!");
         }
         taskMutator.mutate("Task.doLast(Action)", new Runnable() {
             public void run() {
-                getTaskActions().add(wrap(action));
+                getTaskActions().add(wrap(action, actionName));
             }
         });
         return this;
@@ -534,6 +565,11 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         return taskDestroyables;
     }
 
+    @Override
+    public TaskLocalState getLocalState() {
+        return taskLocalState;
+    }
+
     @Internal
     protected ServiceRegistry getServices() {
         return services;
@@ -541,7 +577,9 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     @Override
     public boolean dependsOnTaskDidWork() {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("Task.dependsOnTaskDidWork()");
+        DeprecationLogger.nagUserOfDiscontinuedMethod(
+            "Task.dependsOnTaskDidWork()",
+            "Instead, check the value of \"didWork()\" for each task, or declare the task inputs and outputs and let Gradle decide what needs to be run.");
         TaskDependency dependency = getTaskDependencies();
         for (Task depTask : dependency.getDependencies(this)) {
             if (depTask.getDidWork()) {
@@ -559,7 +597,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         }
         taskMutator.mutate("Task.doFirst(Closure)", new Runnable() {
             public void run() {
-                getTaskActions().add(0, convertClosureToAction(action));
+                getTaskActions().add(0, convertClosureToAction(action, "doFirst {} action"));
             }
         });
         return this;
@@ -573,7 +611,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         }
         taskMutator.mutate("Task.doLast(Closure)", new Runnable() {
             public void run() {
-                getTaskActions().add(convertClosureToAction(action));
+                getTaskActions().add(convertClosureToAction(action, "doLast {} action"));
             }
         });
         return this;
@@ -589,7 +627,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         }
         taskMutator.mutate("Task.leftShift(Closure)", new Runnable() {
             public void run() {
-                getTaskActions().add(taskMutator.leftShift(convertClosureToAction(action)));
+                getTaskActions().add(taskMutator.leftShift(convertClosureToAction(action, "doLast {} action")));
             }
         });
         return this;
@@ -619,26 +657,27 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     @Override
     public void addValidator(TaskValidator validator) {
-        getValidators().add(validator);
+        throw new UnsupportedOperationException();
     }
 
     @Override
     public List<TaskValidator> getValidators() {
-        if (validators == null) {
-            validators = new ArrayList<TaskValidator>();
-        }
-        return validators;
+        throw new UnsupportedOperationException();
     }
 
-    private ContextAwareTaskAction convertClosureToAction(Closure actionClosure) {
-        return new ClosureTaskAction(actionClosure);
+    private ContextAwareTaskAction convertClosureToAction(Closure actionClosure, String actionName) {
+        return new ClosureTaskAction(actionClosure, actionName);
     }
 
     private ContextAwareTaskAction wrap(final Action<? super Task> action) {
+        return wrap(action, "unnamed action");
+    }
+
+    private ContextAwareTaskAction wrap(final Action<? super Task> action, String actionName) {
         if (action instanceof ContextAwareTaskAction) {
             return (ContextAwareTaskAction) action;
         }
-        return new TaskActionWrapper(action);
+        return new TaskActionWrapper(action, actionName);
     }
 
     private static class TaskInfo {
@@ -655,9 +694,11 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
 
     private static class ClosureTaskAction implements ContextAwareTaskAction {
         private final Closure closure;
+        private final String actionName;
 
-        private ClosureTaskAction(Closure closure) {
+        private ClosureTaskAction(Closure closure, String actionName) {
             this.closure = closure;
+            this.actionName = actionName;
         }
 
         @Override
@@ -700,13 +741,25 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         public String getActionClassName() {
             return AbstractTask.getActionClassName(closure);
         }
+
+        @Override
+        public String getDisplayName() {
+            return "Execute " + actionName;
+        }
     }
 
     private static class TaskActionWrapper implements ContextAwareTaskAction {
         private final Action<? super Task> action;
+        private final String maybeActionName;
 
-        public TaskActionWrapper(Action<? super Task> action) {
+        /**
+         * The <i>action name</i> is used to construct a human readable name for
+         * the actions to be used in progress logging. It is only used if
+         * the wrapped action does not already implement {@link Describable}.
+         */
+        public TaskActionWrapper(Action<? super Task> action, String maybeActionName) {
             this.action = action;
+            this.maybeActionName = maybeActionName;
         }
 
         @Override
@@ -774,6 +827,14 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
         public int hashCode() {
             return action != null ? action.hashCode() : 0;
         }
+
+        @Override
+        public String getDisplayName() {
+            if (action instanceof Describable) {
+                return ((Describable) action).getDisplayName();
+            }
+            return "Execute " + maybeActionName;
+        }
     }
 
     private static String getActionClassName(Object action) {
@@ -941,7 +1002,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
      * @since 4.1
      */
     @Incubating
-    protected DirectoryVar newOutputDirectory() {
+    protected DirectoryProperty newOutputDirectory() {
         return getServices().get(TaskFileVarFactory.class).newOutputDirectory(this);
     }
 
@@ -952,7 +1013,7 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
      * @since 4.1
      */
     @Incubating
-    protected RegularFileVar newOutputFile() {
+    protected RegularFileProperty newOutputFile() {
         return getServices().get(TaskFileVarFactory.class).newOutputFile(this);
     }
 
@@ -963,7 +1024,18 @@ public abstract class AbstractTask implements TaskInternal, DynamicObjectAware {
      * @since 4.1
      */
     @Incubating
-    protected RegularFileVar newInputFile() {
+    protected RegularFileProperty newInputFile() {
         return getServices().get(TaskFileVarFactory.class).newInputFile(this);
     }
+
+    /**
+     * Creates a new input directory property for this task.
+     *
+     * @return The property.
+     * @since 4.3
+     */
+    @Incubating
+    protected DirectoryProperty newInputDirectory() {
+        return getServices().get(TaskFileVarFactory.class).newInputDirectory(this);
+    }
 }
