@@ -19,11 +19,13 @@ package org.gradle.api.internal.file;
 import org.gradle.api.Project;
 import org.gradle.api.Task;
 import org.gradle.api.file.Directory;
+import org.gradle.api.file.DirectoryProperty;
 import org.gradle.api.file.DirectoryVar;
 import org.gradle.api.file.FileSystemLocation;
 import org.gradle.api.file.FileTree;
 import org.gradle.api.file.ProjectLayout;
 import org.gradle.api.file.RegularFile;
+import org.gradle.api.file.RegularFileProperty;
 import org.gradle.api.file.RegularFileVar;
 import org.gradle.api.internal.provider.AbstractCombiningProvider;
 import org.gradle.api.internal.provider.AbstractMappingProvider;
@@ -35,6 +37,7 @@ import org.gradle.api.internal.tasks.TaskDependencyResolveContext;
 import org.gradle.api.provider.Provider;
 import org.gradle.internal.Factory;
 import org.gradle.internal.file.PathToFileResolver;
+import org.gradle.util.DeprecationLogger;
 
 import javax.annotation.Nullable;
 import java.io.File;
@@ -54,32 +57,44 @@ public class DefaultProjectLayout implements ProjectLayout, TaskFileVarFactory {
     }
 
     @Override
-    public DirectoryVar getBuildDirectory() {
+    public DirectoryProperty getBuildDirectory() {
         return buildDir;
     }
 
     @Override
     public DirectoryVar newDirectoryVar() {
+        DeprecationLogger.nagUserOfReplacedMethod("ProjectLayout.newDirectoryVar()", "ProjectLayout.directoryProperty()");
+        return directoryProperty();
+    }
+
+    @Override
+    public DirectoryVar directoryProperty() {
         return new DefaultDirectoryVar(projectDir.fileResolver);
     }
 
     @Override
     public RegularFileVar newFileVar() {
+        DeprecationLogger.nagUserOfReplacedMethod("ProjectLayout.newFileVar()", "ProjectLayout.fileProperty()");
+        return fileProperty();
+    }
+
+    @Override
+    public RegularFileVar fileProperty() {
         return new DefaultRegularFileVar(projectDir.fileResolver);
     }
 
     @Override
-    public DirectoryVar newOutputDirectory(Task producer) {
+    public DirectoryProperty newOutputDirectory(Task producer) {
         return new BuildableDirectoryVar(projectDir.fileResolver, producer);
     }
 
     @Override
-    public RegularFileVar newOutputFile(Task producer) {
+    public RegularFileProperty newOutputFile(Task producer) {
         return new BuildableRegularFileVar(projectDir.fileResolver, producer);
     }
 
     @Override
-    public RegularFileVar newInputFile(final Task consumer) {
+    public RegularFileProperty newInputFile(final Task consumer) {
         final DefaultRegularFileVar fileVar = new DefaultRegularFileVar(projectDir.fileResolver);
         consumer.dependsOn(new AbstractTaskDependency() {
             @Override
@@ -90,6 +105,18 @@ public class DefaultProjectLayout implements ProjectLayout, TaskFileVarFactory {
         return fileVar;
     }
 
+    @Override
+    public DirectoryProperty newInputDirectory(final Task consumer) {
+        final DefaultDirectoryVar directoryVar = new DefaultDirectoryVar(projectDir.fileResolver);
+        consumer.dependsOn(new AbstractTaskDependency() {
+            @Override
+            public void visitDependencies(TaskDependencyResolveContext context) {
+                directoryVar.visitDependencies(context);
+            }
+        });
+        return directoryVar;
+    }
+
     @Override
     public Provider<RegularFile> file(Provider<File> provider) {
         return new AbstractMappingProvider<RegularFile, File>(RegularFile.class, provider) {
@@ -205,6 +232,15 @@ public class DefaultProjectLayout implements ProjectLayout, TaskFileVarFactory {
             }
         }
 
+        @Override
+        public void setFromAnyValue(Object object) {
+            if (object instanceof File) {
+                set((File) object);
+            } else {
+                super.setFromAnyValue(object);
+            }
+        }
+
         @Override
         public Provider<File> getAsFile() {
             return new ToFileProvider(this);
@@ -282,6 +318,16 @@ public class DefaultProjectLayout implements ProjectLayout, TaskFileVarFactory {
             set(new ResolvingDirectory(resolver, value, null));
         }
 
+        @Override
+        public void setFromAnyValue(Object object) {
+            if (object instanceof File) {
+                File file = (File) object;
+                set(file);
+            } else {
+                super.setFromAnyValue(object);
+            }
+        }
+
         @Override
         public void visitDependencies(TaskDependencyResolveContext context) {
             if (getProvider() instanceof TaskDependencyContainer) {
