@@ -19,9 +19,9 @@ package org.gradle.internal.component.external.model;
 import org.gradle.internal.component.model.ModuleSource;
 
 /**
- * Meta-data for a module version resolved from a Maven repository.
+ * Meta-data for a component resolved from a Maven repository.
  */
-public interface MavenModuleResolveMetadata extends ModuleComponentResolveMetadata {
+public interface MavenModuleResolveMetadata extends ModuleComponentResolveMetadata, ComponentVariantResolveMetadata {
     /**
      * {@inheritDoc}
      */
